/////////////////////////////////////////////////////////////////////////////
// Gdi.h
//
// Types and defines for Tasking GDI interface routines.

#ifndef __GDI_H
#define __GDI_H

#pragma pack(push, 8)

#define GDI_VERSION_HIGH    1
#define GDI_VERSION_LOW     2
#define GDI_VERSION_POINT   6

typedef unsigned char   DiBoolT;
typedef unsigned char   DiUCharT;
typedef int             DiInt32T;
typedef unsigned int    DiUInt32T;
typedef char*           DiStringT;

#define cbDiValueT 8

typedef struct
{
    DiUCharT val[cbDiValueT];

} DiValueT, *pDiValueT;

// Convert big endian DiValueT value to 32-bit little endian value
#define DIVALUE32(x) ((x).val[7]|((x).val[6]<<8)|((x).val[5]<<16)|((x).val[4]<<24))

// Convert little endian 32-bit value to big endian DiValueT
#define DISETVALUE32(x,n)                 \
    if (1)                                \
    {                                     \
        (x).val[7] = *((char*)(&(n)));    \
        (x).val[6] = *((char*)(&(n))+1);  \
        (x).val[5] = *((char*)(&(n))+2);  \
        (x).val[4] = *((char*)(&(n))+3);  \
        (x).val[3] = 0;                   \
        (x).val[2] = 0;                   \
        (x).val[1] = 0;                   \
        (x).val[0] = 0;                   \
    }

typedef struct
{
    DiValueT v;

} DiLAddrT, *pDiLAddrT;

typedef DiUInt32T DiMemSpaceT;

typedef struct
{
    DiMemSpaceT dmsMemSpace;
    DiLAddrT    dlaLinAddress;

} DiAddrT;

#define cbDiMaxBitsPerMemT      64
#define cbDiMaxUCharTsPerMemT   ((cbDiMaxBitsPerMemT+7)/8)

typedef struct
{
    DiUCharT val[cbDiMaxUCharTsPerMemT];

} DiMemT, *pDiMemT;

typedef DiUInt32T DiValueStatusT, *pDiValueStatusT;

#define DI_VS_DEFINED       0x0000
#define DI_VS_FLOATING      0x0001
#define DI_VS_UNINITIALIZED 0x0002
#define DI_VS_UNKNOWN       0x0003
#define DI_VS_READONLY      0x0004
#define DI_VS_WRITEONLY     0x0005
#define DI_VS_NO_MEMORY     0x0006

typedef struct
{
    DiMemT          dmValue;
    DiValueStatusT  dvsStatus;

} DiMemValueT, *pDiMemValueT;

typedef struct
{
    DiValueT        dvValue;
    DiValueStatusT  dvsStatus;

} DiRegisterValueT, *pDiRegisterValueT;

typedef struct
{
    DiStringT   szMemSpaceName;
    DiUInt32T   dnMemSpaceId;
    DiBoolT     fGdiSupport;

} DiMemorySpaceInfoT;

typedef struct
{
    DiStringT   szRegName;
    DiUInt32T   dnRegNumber;
    DiBoolT     fGdiSupport;

} DiRegisterInfoT;

typedef enum
{
    DI_OK                   = 0,
    DI_ERR_STATE            = 1,
    DI_ERR_PARAM            = 2,
    DI_ERR_COMMUNICATION    = 3,
    DI_ERR_NOTSUPPORTED     = 4,
    DI_ERR_NONFATAL         = 5,
    DI_ERR_CANCEL           = 6,
    DI_ERR_FATAL            = 7

} DiReturnT;

typedef DiUInt32T DiBpTypeT, *pDiBpTypeT;

#define DI_BPT_EXECUTE      0x0001
#define DI_BPT_READ         0x0002
#define DI_BPT_WRITE        0x0004
#define DI_BPT_VALUE        0x0008
#define DI_BPT_TIME         0x0010
#define DI_BPT_CYCLES       0x0020
#define DI_BPT_INSTRUCTIONS 0x0040
#define DI_BPT_EXTERN       0x0080

typedef enum
{
    DI_BPM_HARDWARE = 0,
    DI_BPM_SOFTWARE = 1,
    DI_BPM_NONE = 2

} DiBpMethodT, *pDiBpMethodT;

typedef struct
{
    DiBpTypeT dbtBpType;
    union
    {
        struct
        {
            DiAddrT     daBp;
            DiUInt32T   dnSize;
            DiBpMethodT dbmBpMethod;
        } exe;
        struct
        {
            DiAddrT     daBp;
            DiUInt32T   dnSize;
            DiValueT    dnValue;
        } rwv;
        /* Only EXE breakpoints are supported
        struct
        {
            DiTimeT dtTime;
        } tim;
        struct
        {
            DiValueT dvCycle;
        } cyc;
        struct
        {
            DiValueT dvInstr;
        } inst;
        */
    } u;

} DiBpT, *pDiBpT;

typedef struct
{
    DiUInt32T   dnBpId;
    DiBpT       dbBp;

} DiBpResultT, *pDiBpResultT;

typedef DiUInt32T DiExitCauseT, *pDiExitCauseT;

#define DI_WAIT_RUNNING         0x0000
#define DI_WAIT_SINGLESTEP      0x0001
#define DI_WAIT_CONTINUEUNTIL   0x0002
#define DI_WAIT_CODEBP          0x0004
#define DI_WAIT_DATABP          0x0008
#define DI_WAIT_CYCLES          0x0010
#define DI_WAIT_TIME            0x0020
#define DI_WAIT_INSTRUCTIONS    0x0040
#define DI_WAIT_USER            0x0080
#define DI_WAIT_EXTERNAL        0x0100
#define DI_WAIT_MISCELLANEOUS   0x0200
#define DI_WAIT_UNKNOWN         0x0400

typedef struct
{
    DiExitCauseT    dscCause;
    DiUInt32T       dwBpId;
    DiStringT       szReason;

} DiExitStatusT, *pDiExitStatusT;

typedef DiUInt32T DiCommChannelT, *pDiCommChannelT;

#define DI_COMM_NONE        0x0000
#define DI_COMM_RS232       0x0001
#define DI_COMM_RS422       0x0002
#define DI_COMM_RS485       0x0004
#define DI_COMM_TCPIP       0x0008
#define DI_COMM_IPXSPX      0x0010
#define DI_COMM_IPC         0x0020
#define DI_COMM_RPC         0x0040
#define DI_COMM_SCSI        0x0080
#define DI_COMM_CAN         0x0100
#define DI_COMM_I2C         0x0200
#define DI_COMM_PROFIBUS    0x0400
#define DI_COMM_INTERBUS    0x0800
#define DI_COMM_TAP         0x1000
#define DI_COMM_DEVICE_FILE 0x2000
#define DI_COMM_ISA_BUS     0x4000
#define DI_COMM_PARALLEL    0x8000

typedef DiUInt32T DiAbsFileT, *pDiAbsFileT;

#define DI_ABSF_NONE                0x0000
#define DI_ABSF_FILENAME            0x0001
#define DI_ABSF_TIOF695             0x0002
#define DI_ABSF_HPMRI695            0x0004
#define DI_ABSF_MOTOROLA_S          0x0008
#define DI_ABSF_INTEL_HEX           0x0010
#define DI_ABSF_TEKTRONIC_HEX       0x0020
#define DI_ABSF_EXT_TEKTRONIC_HEX   0x0040
#define DI_ABSF_BINARY              0x0080

typedef struct
{
    DiAbsFileT  dafAbsFileFormat;
    DiUInt32T   dnBufferSize;
    DiAddrT     daAddress;

} DiDownloadFormatT, *pDiDownloadFormatT;

typedef DiUInt32T DiCallbackT, *pDiCallbackT;

#define DI_CB_NONE          0x0000
#define DI_CB_BREAK         0x0001
#define DI_CB_EXCEPTION     0x0002
#define DI_CB_SYNCHRONIZE   0x0004
#define DI_CB_EXTERNBPSET   0x0008
#define DI_CB_MISCELLANEOUS 0x0010
#define DI_CB_EXITREQUEST   0x0020
#define DI_CB_LOG           0x0040
#define DI_CB_APPLI_INPUT   0x0080
#define DI_CB_APPLI_OUTPUT  0x0100
#define DI_CB_EVAL_EXPR     0x0200
#define DI_CB_ADDRTOSYMBOL  0x0400
#define DI_CB_DEBUG         0x0800
#define DI_CB_RESOURCES     0x1000
#define DI_CB_FEEDBACK      0x2000

typedef struct
{
    DiBoolT     fBgSupport;
    DiMemSpaceT dmsMemSpace;

} DiBackGroundT, *pDiBackGroundT;

typedef enum
{
    DI_UNKNOWN_ENDIANNESS   = 0,
    DI_BIG_ENDIAN           = 1,
    DI_LITTLE_ENDIAN        = 2

} DiEndianT;

typedef struct
{
    DiStringT szIdentification;
    DiStringT szVersion;
    DiUInt32T dnVersion;
    DiStringT *pszConfig;
    DiUInt32T dnConfigArrayItems;
    DiCommChannelT dccIOChannel;
    DiBoolT fMemorySetMapAvailable;
    DiBoolT fMemorySetCpuMapAvailable;
    DiStringT *pszMemoryType;
    DiUInt32T dnMemTypeArrayItems;
    DiBoolT fEnableReadaheadCache;
    DiBoolT fTimerInCycles;
    DiUInt32T dnTimerResolutionMantissa;
    DiInt32T dnTimerResolutionExponent;
    DiDownloadFormatT ddfDownloadFormat;
    DiDownloadFormatT ddfAuxDownloadFormat;
    DiBoolT fAuxiliaryDownloadPathAvailable;
    DiCallbackT dcCallback;
    DiBoolT fRegisterClassSupport;
    DiBoolT fSingleStepSupport;
    DiBoolT fContinueUntilSupport;
    DiBoolT fContinueBackgroundSupport;
    DiUInt32T dnNrCodeBpAvailable;
    DiUInt32T dnNrDataBpAvailable;
    DiBoolT fExecFromCodeBp;
    DiBoolT fExecFromDataBp;
    DiBoolT fUnifiedBpLogic;
    DiBoolT fExecCycleCounterAvailable;
    DiBoolT fExecTimeAvailable;
    DiBoolT fInstrTraceAvailable;
    DiBoolT fRawTraceAvailable;
    DiBoolT fCoverageAvailable;
    DiBoolT fProfilingAvailable;
    DiBoolT fStateSaveRestoreAvailable;
    DiUInt32T dnStateStoreMaxIndex;
    DiBackGroundT *pdbgBackground;
    DiUInt32T dnBackgroundArrayItems;
    DiBoolT fDirectDiAccessAvailable;
    DiBoolT fApplicationIOAvailable;
    DiBoolT fKernelAware;
    DiBoolT fMeeAvailable;
    DiUInt32T dnNrCpusAvailable;
    DiEndianT deWordEndianness;
    DiUInt32T dnNrHardWareCodeBpAvailable;
    DiBoolT fCodeHardWareBpSkids;
    void *pReserved;

} DiFeaturesT, *pDiFeaturesT;

typedef DiUInt32T DiCoverageT, *pDiCoverageT;

#define DI_COV_UNKNOWN  0x0000
#define DI_COV_EXECUTE  0x0001
#define DI_COV_READ     0x0002
#define DI_COV_WRITE    0x0004
#define DI_COV_ACCESS   0x0008

typedef enum
{
    DI_RS232_NONE = 0,
    DI_RS232_RTS_CTS = 1,
    DI_RS232_XON_XOFF = 2

} DiRs232HandshakeT;

typedef struct
{
    DiCommChannelT  dccType;
    DiBoolT         fCheckConnection;
    union
    {
        struct
        {
            DiStringT           szPortName;
            DiUInt32T           dnBaudRate;
            DiRs232HandshakeT   drhHandshake;
            DiUInt32T           dnTimeoutFactor;
            DiBoolT             fExclusiveAccess;
        } Rs232;
    } u;

} DiCommSetupT, *pDiCommSetupT;

typedef enum
{
    DI_UI_SOURCE = 0,
    DI_UI_ADDRESS = 1,
    DI_UI_SYMBOL = 2,
    DI_UI_STRING = 3

} DiDebugInfoT, *pDiDebugInfoT;

typedef struct
{
    DiDebugInfoT ddiType;
    union
    {
        struct
        {
            DiStringT   szSrcFileName;
            DiUInt32T   dnSrcLineNumber;
            DiAddrT     daSrcAddress;
        } source;
        struct
        {
            DiAddrT     daAddress;
        } instr;
        struct
        {
            DiStringT   szSymbolName;
            DiAddrT     daAddress;
        } symbol;
        struct
        {
            DiStringT   szString;
        } string;
    } u;

} DiUserInfoT;

typedef void (*CallbackF)(DiCallbackT, ...);

typedef struct
{
    DiAddrT     daStart;
    DiUInt32T   dnSize;

} DiAddrRangeT, *pDiAddrRangeT;

typedef struct
{
    DiAddrRangeT    darCpu;
    DiAddrT         daPhysMem;

} DiMemoryToCpuItemT, *pDiMemoryToCpuItemT;

typedef struct
{
    DiUInt32T           dnNrItems;
    DiMemoryToCpuItemT* pdmtciItems;

} DiMemoryToCpuT, *pDiMemoryToCpuT;

typedef DiUInt32T DiAccessT, *pDiAccessT;

#define DI_ACC_NONE     0x0000
#define DI_ACC_EXECUTE  0x0001
#define DI_ACC_READ     0x0002
#define DI_ACC_WRITE    0x0004

typedef struct
{
    DiAddrRangeT    darRange;
    DiStringT       szMemType;
    DiAccessT       daAccess;

} DiMemoryMapItemT, *pDiMemoryMapItemT;

typedef struct
{
    DiUInt32T           dnNrItems;
    DiMemoryMapItemT*   pdmmiItems;

} DiMemoryMapT, *pDiMemoryMapT;

typedef struct
{
    DiUInt32T dnNrItems;
    DiStringT *pszName;

} DiExecEnvT, *pDiExecEnvT;

typedef struct
{
    DiStringT szMenuTitle;
    DiStringT szDiCommand;

} DiMenuItemT, *pDiMenuItemT;

typedef enum
{
    DI_DNLD_INIT = 0,
    DI_DNLD_WRITE = 1,
    DI_DNLD_TERMINATE = 2,
    DI_DNLD_ABORT = 3

} DiDownloadCommandT, *pDiDownloadCommandT;

typedef struct
{
    DiValueT v;

} DiTimeT, *pDiTimeT;

typedef struct
{
    DiAddrRangeT    darRange;
    DiTimeT         dtTime;

} DiProfT, *pDiProfT;

typedef struct
{
    DiUInt32T dnEntries;
    pDiProfT *pdpProf;

} DiProfileT, *pDiProfileT;

typedef struct
{
    DiAddrT daPc;
    DiTimeT dtTime;

} DiInstrTraceItemT, *pDiInstrTraceItemT;

typedef struct
{
    DiUInt32T           dnNrInstr;
    DiInstrTraceItemT*  pditiItems;

} DiInstrTraceT, *pDiInstrTraceT;

typedef enum
{
    DI_TRACE_INSTR = 0,
    DI_TRACE_RAW = 1,
    DI_TRACE_RAW1 = 2,
    DI_TRACE_RAW2 = 3

} DiTraceTypeT, *pDiTraceTypeT;

typedef struct
{
    DiUInt32T   dnNrNewFrames;
    DiBoolT     fTraceBufferOverflow;

} DiNewFramesT, *pDiNewFramesT;

#pragma pack(pop)

#endif
