///////////////////////////////////////////////////////////////////////
// GdiUser.h
//
// Header file for applications using the GDI interface. In one of the
// project source files, define GDI_USER_MAIN before including this
// header to declare routine wrappers.

#ifndef __GDI_USER_H
#define __GDI_USER_H

#include "Gdi.h"
#include <assert.h>

// GDI vars are actually declared if GDI_USER_MAIN is defined
#ifdef GDI_USER_MAIN
#define EXTERN
#else
#define EXTERN extern
#endif

// DiGdiOpen
typedef DiReturnT (*DiGdiOpenPtr)(DiUInt32T, DiUInt32T, DiUInt32T, DiStringT[],
                            void (*)(void));
EXTERN DiGdiOpenPtr g_pfnDiGdiOpen;
#define DiGdiOpen g_pfnDiGdiOpen

// DiGdiVersion
typedef DiReturnT (*DiGdiVersionPtr)(DiUInt32T*);
EXTERN DiGdiVersionPtr g_pfnDiGdiVersion;
#define DiGdiVersion g_pfnDiGdiVersion

// DiGdiClose
typedef DiReturnT (*DiGdiClosePtr)(DiBoolT);
EXTERN DiGdiClosePtr g_pfnDiGdiClose;
#define DiGdiClose g_pfnDiGdiClose

// DiGdiGetFeatures
typedef DiReturnT (*DiGdiGetFeaturesPtr)(pDiFeaturesT);
EXTERN DiGdiGetFeaturesPtr g_pfnDiGdiGetFeatures;
#define DiGdiGetFeatures g_pfnDiGdiGetFeatures

// DiGdiInitIO
typedef DiReturnT (*DiGdiInitIOPtr)(pDiCommSetupT);
EXTERN DiGdiInitIOPtr g_pfnDiGdiInitIO;
#define DiGdiInitIO g_pfnDiGdiInitIO

// DiGdiInitRegisterMap
typedef DiReturnT (*DiGdiInitRegisterMapPtr)(DiUInt32T, DiRegisterInfoT*, DiUInt32T);
EXTERN DiGdiInitRegisterMapPtr g_pfnDiGdiInitRegisterMap;
#define DiGdiInitRegisterMap g_pfnDiGdiInitRegisterMap

// DiGdiInitMemorySpace
typedef DiReturnT (*DiGdiInitMemorySpaceMapPtr)(DiUInt32T, DiMemorySpaceInfoT*);
EXTERN DiGdiInitMemorySpaceMapPtr g_pfnDiGdiInitMemorySpaceMap;
#define DiGdiInitMemorySpaceMap g_pfnDiGdiInitMemorySpaceMap

// DiRegisterRead
typedef DiReturnT (*DiRegisterReadPtr)(DiUInt32T, pDiRegisterValueT);
EXTERN DiRegisterReadPtr g_pfnDiRegisterRead;
#define DiRegisterRead g_pfnDiRegisterRead

// DiRegisterWrite
typedef DiReturnT (*DiRegisterWritePtr)(DiUInt32T, DiRegisterValueT);
EXTERN DiRegisterWritePtr g_pfnDiRegisterWrite;
#define DiRegisterWrite g_pfnDiRegisterWrite

// DiMemoryRead
typedef DiReturnT (*DiMemoryReadPtr)(DiAddrT, DiMemValueT*, DiUInt32T);
EXTERN DiMemoryReadPtr g_pfnDiMemoryRead;
#define DiMemoryRead g_pfnDiMemoryRead

// DiMemoryWrite
typedef DiReturnT (*DiMemoryWritePtr)(DiAddrT, DiMemValueT*, DiUInt32T);
EXTERN DiMemoryWritePtr g_pfnDiMemoryWrite;
#define DiMemoryWrite g_pfnDiMemoryWrite

// DiExecResetChild
typedef DiReturnT (*DiExecResetChildPtr)(void);
EXTERN DiExecResetChildPtr g_pfnDiExecResetChild;
#define DiExecResetChild g_pfnDiExecResetChild

// DiExecSingleStep
typedef DiReturnT (*DiExecSingleStepPtr)(DiUInt32T);
EXTERN DiExecSingleStepPtr g_pfnDiExecSingleStep;
#define DiExecSingleStep g_pfnDiExecSingleStep

// DiExecContinue
typedef DiReturnT (*DiExecContinuePtr)(void);
EXTERN DiExecContinuePtr g_pfnDiExecContinue;
#define DiExecContinue g_pfnDiExecContinue

// DiExecContinueUntil
typedef DiReturnT (*DiExecContinueUntilPtr)(DiAddrT addrUntil);
EXTERN DiExecContinueUntilPtr g_pfnDiExecContinueUntil;
#define DiExecContinueUntil g_pfnDiExecContinueUntil

// DiExecStop
typedef DiReturnT (*DiExecStopPtr)(void);
EXTERN DiExecStopPtr g_pfnDiExecStop;
#define DiExecStop g_pfnDiExecStop

// DiBreakpointSet
typedef DiReturnT (*DiBreakpointSetPtr)(DiBpResultT*, DiBpT);
EXTERN DiBreakpointSetPtr g_pfnDiBreakpointSet;
#define DiBreakpointSet g_pfnDiBreakpointSet

// DiBreakpointClear
typedef DiReturnT (*DiBreakpointClearPtr)(DiUInt32T);
EXTERN DiBreakpointClearPtr g_pfnDiBreakpointClear;
#define DiBreakpointClear g_pfnDiBreakpointClear

// DiBreakpointClearAll
typedef DiReturnT (*DiBreakpointClearAllPtr)(void);
EXTERN DiBreakpointClearAllPtr g_pfnDiBreakpointClearAll;
#define DiBreakpointClearAll g_pfnDiBreakpointClearAll

// DiExecGetStatus
typedef DiReturnT (*DiExecGetStatusPtr)(DiExitStatusT* pdesExitStatus);
EXTERN DiExecGetStatusPtr g_pfnDiExecGetStatus;
#define DiExecGetStatus g_pfnDiExecGetStatus

#endif
