/////////////////////////////////////////////////////////////////////////////
// MemoryReadDlg.c

#include "Windows.h"
#include "GdiExample.h"

int g_nMemSel         = 0;
int g_nMemAddress     = 0;
static int g_cbCount  = 0x20;

BOOL CALLBACK MemoryReadDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam,
								LPARAM lParam)
{
    HWND hwndList;
    DiReturnT fRet;
    char szTemp[20];
    DiAddrT address;
    DiMemValueT memValues[32];
    int i, j, nLine, niSave;
    char* psz;

	switch (uMsg)
	{
	case WM_INITDIALOG:

        // Populate memory space list
        hwndList = GetDlgItem(hDlg, IDC_MEMORY_SPACE);
        for (i = 0; i < NUM_MEMORY_SPACES; i++)
            SendMessage(hwndList, CB_ADDSTRING, 0, (LPARAM) g_memorySpaces[i].szMemSpaceName);
        SendMessage(hwndList, CB_SETCURSEL, g_nMemSel, 0);       

        // Initialize edit fields
        itoa(g_nMemAddress, szTemp, 16);
        SetDlgItemText(hDlg, IDC_ADDRESS, szTemp);
        itoa(g_cbCount, szTemp, 16);
        SetDlgItemText(hDlg, IDC_BYTE_COUNT, szTemp);
		return TRUE;

	case WM_COMMAND:

		switch (LOWORD(wParam))
		{
		case IDC_READ:

            // Clear display
            FreeOutputLines();
            InvalidateRect(g_hMainWnd, NULL, TRUE);

            // Get current selection in memory space list
            hwndList = GetDlgItem(hDlg, IDC_MEMORY_SPACE);
            g_nMemSel = SendMessage(hwndList, CB_GETCURSEL, 0, 0);
            if (g_nMemSel == CB_ERR)
                return FALSE;

            // Map list index to memory space ID
            ASSERT(g_nMemSel < NUM_MEMORY_SPACES);
            address.dmsMemSpace = g_memorySpaces[g_nMemSel].dnMemSpaceId;

            // Scan address from edit field
            GetDlgItemText(hDlg, IDC_ADDRESS, szTemp, sizeof(szTemp));
            if (sscanf(szTemp, "%x", &g_nMemAddress) != 1)
            {
                MessageBox(NULL, "Invalid address.", "Error", MB_OK|MB_ICONSTOP);
                return FALSE;
            }

            DISETVALUE32(address.dlaLinAddress.v, g_nMemAddress);

            // Scan byte count from edit field
            GetDlgItemText(hDlg, IDC_BYTE_COUNT, szTemp, sizeof(szTemp));
            if (sscanf(szTemp, "%x", &g_cbCount) != 1)
            {
                MessageBox(NULL, "Invalid byte count.", "Error", MB_OK|MB_ICONSTOP);
                return FALSE;
            }

            // Limit byte count
            if (g_cbCount <= 0)
                g_cbCount = 1;
            else if (g_cbCount > sizeof(memValues))
                g_cbCount = sizeof(memValues);

            // Read memory
            fRet = DiMemoryRead(address, memValues, g_cbCount);
            if (fRet != DI_OK)
                MessageBox(NULL, "Error reading memory.", "Error", MB_OK|MB_ICONSTOP);
            else
            {
                for (i = 0, nLine = 0; i < g_cbCount; nLine++)
                {
                    // Allocate next line for displaying 8 bytes
                    g_outputLines[nLine] = malloc(256);
                    psz = g_outputLines[nLine];
                    *psz = 0;

                    psz += sprintf(psz, "%08X  ", g_nMemAddress + i);

                    // Print up to 8 values into new line
                    niSave = i;
                    for (j = 0; j < 8 && i < g_cbCount; j++, i++)
                        psz += sprintf(psz, "%02X ", memValues[i].dmValue.val[7]);

                    // Pad with spaces for lines with < 8 values
                    for (; j < 8; j++)
                    {
                        strcat(psz, "   ");
                        psz += 3;
                    }

                    // Add an extra space
                    strcat(psz, " ");
                    psz++;

                    // Print as characters
                    i = niSave;
                    for (j = 0; j < 8 && i < g_cbCount; j++, i++)
                        psz += sprintf(psz, "%c", memValues[i].dmValue.val[7]);
                }
            }
            
            return TRUE;

		case IDCANCEL:

			EndDialog(hDlg, 0);
			return TRUE;
		}

		break;

	}

	return FALSE;
}
